
!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/11/2010 15:13:54.92'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; width: self scrollBarThickness; changeTableLayout; listDirection: #topToBottom.
	horizontalHeader := Morph new color: Color transparent; height: self scrollBarThickness; changeTableLayout; listDirection: #leftToRight.
	vHeaderScroller := TransformMorph new color: Color transparent.
	hHeaderScroller := TransformMorph new color: Color transparent.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: UiTreeViewport new.
	self connect: self viewport signal: #doubleClicked: toSelector: #toggleIndex:.
	self connect: self viewport signal: #decorationClicked: toSelector: #toggleIndex:.
	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	itemDelegates at: #default put: UiSimpleItemDelegate new.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller addMorph: horizontalHeader.

	self
		"addMorph: vHeaderScroller;"
		addMorph: hHeaderScroller.
	
	self resizeScroller.! !

Object subclass: #UiViewSelection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

UiViewSelection subclass: #UiViewExtendedSelection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

UiViewSelection subclass: #UiViewMultiSelection
	instanceVariableNames: 'cells'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiViewMultiSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 14:58:34.066'!
add: aCellIndex event: aUserInputEvent

	"Invert the cell."
	self cells
		remove: aCellIndex
		ifAbsent: [self cells add: aCellIndex].
		
	self selectionChanged.! !

!UiViewMultiSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 14:56:08.346'!
cells

	^ cells ifNil: [cells := Set new]! !

UiViewSelection subclass: #UiViewNullSelection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiViewNullSelection commentStamp: 'mt 10/11/2010 14:09:45.699' prior: 0!
No selection of cells possible.!

!UiViewNullSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 14:09:04.349'!
add: aCellIndex event: aUserInputEvent

	"Ignore."! !

!UiViewNullSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 14:08:50.077'!
cells

	^ #()! !

!UiViewSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 08:57:38.881'!
add: aCellIndex! !

!UiViewSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 09:07:25.205'!
add: aCellIndex event: aUserInputEvent! !

!UiViewSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 08:56:27.565'!
addRangeFrom: topLeftCell to: bottomRightCell! !

!UiViewSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 08:57:25.839'!
addRangeTo: targetCell! !

!UiViewSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 09:13:15.101'!
cells
	"Return an array with all selected cells."
	
	self subclassResponsibility.! !

!UiViewSelection methodsFor: 'signals' stamp: 'mt 10/11/2010 09:12:03.242'!
selectionChanged

	self emit.! !

UiViewSelection subclass: #UiViewSingleSelection
	instanceVariableNames: 'selectedCell'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiViewSingleSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 09:21:51.677'!
add: aCellIndex event: aUserInputEvent

	selectedCell := aCellIndex.
	self selectionChanged.! !

!UiViewSingleSelection methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 14:35:26.499'!
cells

	^ selectedCell
		ifNil: [#()]
		ifNotNil: [{selectedCell}]! !

Morph subclass: #UiTreeViewport
	instanceVariableNames: 'rows columns currentCell hoveredCell selectionColor selection selectionMode selectionBehavior indent verticalHeader horizontalHeader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 15:19:58.95'!
click: evt

	| cell row item |
	cell := self cellIndexFromPoint: evt position.
	cell ifNil: [^ self].
	
	row := rows at: cell y.
	item := row items first.
	
	"Recognize a click on the decoration."
	(evt position x < item left
		and: [evt position x >= (item left - self indent)])
		ifTrue: [self decorationClicked: (item valueOfProperty: #modelIndex)].! !

!UiTreeViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/11/2010 09:20:27.893'!
columnBoundsFor: aColIndex

	| col |
	col := columns at: aColIndex.
	
	^ (self leftForColumn: col) @ self top
		extent: col width @ self height! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:08:10.466'!
currentCell

	^ currentCell! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:08:18.971'!
currentCell: aPoint

	currentCell := aPoint.! !

!UiTreeViewport methodsFor: 'signals' stamp: 'mt 10/11/2010 15:09:27.235'!
decorationClicked: aModelIndex
	"Whenever the user clicks the expand/collapse decoration in front of an item."
	
	self emit.! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:27:00.022'!
drawCurrentCellOn: aCanvas

	self currentCell ifNil: [^ self].
	
	self selectionBehavior = #selectCells ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self cellBoundsFor: self currentCell)
			color: self selectionColor darker.
		^ self].
		
	self selectionBehavior = #selectRows ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self rowBoundsFor: self currentCell y)
			color: self selectionColor darker.
		^ self].
	
	self selectionBehavior = #selectColumns ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self columnBoundsFor: self currentCell x)
			color: self selectionColor darker.
		^ self]! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:34:39.295'!
drawHighlightOn: aCanvas bounds: drawBounds color: aColor
	
	| gradient |

	gradient := (GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (aColor adjustBrightness: 0.05).
			0.6 -> (aColor darker)})
		origin: drawBounds topLeft;
		direction: 0 @ drawBounds height.
		
	aCanvas
		fillRoundRect: (drawBounds insetBy: (1@1 corner: 0@0))
		radius: 3
		fillStyle: gradient;
		frameRoundRect: (drawBounds outsetBy: (0@0 corner: 1@1))
		radius: 3
		width: 1
		color: aColor muchDarker.! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:41:13.054'!
drawHoveredCellOn: aCanvas

	self hoveredCell ifNil: [^ self].
	
	self selectionBehavior = #selectCells ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self cellBoundsFor: self hoveredCell)
			color: self selectionColor muchLighter.
		^ self].
		
	self selectionBehavior = #selectRows ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self rowBoundsFor: self hoveredCell y)
			color: self selectionColor muchLighter.
		^ self].
	
	self selectionBehavior = #selectColumns ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self columnBoundsFor: self hoveredCell x)
			color: self selectionColor muchLighter.
		^ self]! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:42:20.26'!
drawOn: aCanvas
	
	self
		drawHoveredCellOn: aCanvas;
		drawSelectionOn: aCanvas;
		drawCurrentCellOn: aCanvas;
		drawTreeStructureOn: aCanvas.
	super drawOn: aCanvas.! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 09:25:33.057'!
drawSelectionOn: aCanvas

	self selectionBehavior = #selectCells ifTrue: [
		self selection cells do: [:cell |
			self
				drawSelectionOn: aCanvas
				bounds: (self cellBoundsFor: cell)	].
			^ self].
		
	self selectionBehavior = #selectRows ifTrue: [
		(self selection cells collect: [:cell | cell y]) asSet do: [:rowIndex |
			self
				drawSelectionOn: aCanvas
				bounds: (self rowBoundsFor: rowIndex)].
		^ self].
	
	self selectionBehavior = #selectColumns ifTrue: [
		(self selection cells collect: [:cell | cell x]) asSet do: [:colIndex |
			self
				drawSelectionOn: aCanvas
				bounds: (self columnBoundsFor: colIndex)].
		^ self]! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:17:16.138'!
drawSelectionOn: aCanvas bounds: drawBounds
	
	self
		drawHighlightOn: aCanvas
		bounds: drawBounds
		color: self selectionColor.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:52:42.129'!
handleMouseMove: anEvent

	super handleMouseMove: anEvent.
	
	anEvent wasHandled not
	"	and: [anEvent hand mouseFocus == self])"
		ifTrue: [self mouseMove: anEvent. anEvent wasHandled: true]! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:46:59'!
handlesMouseOver: evt

	^ true! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:39:13.856'!
hoveredCell

	^ hoveredCell! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:39:21.601'!
hoveredCell: aPoint

	hoveredCell := aPoint.
	self redrawLater.! !

!UiTreeViewport methodsFor: 'initialization' stamp: 'mt 10/11/2010 09:04:54.243'!
initialize

	super initialize.
	self color: Color transparent.
		
	rows := OrderedCollection new.
	columns := OrderedCollection new.
	
	self selectionMode: UiViewSingleSelection.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 15:06:51.98'!
mouseDown: evt

	self currentCell: (self cellIndexFromPoint: evt position).
	self selection
		add: self currentCell
		event: evt.
	(self selection cells includes: self currentCell)
		ifFalse: [self currentCell: nil].

	evt hand
		waitForClicksOrDrag: self
		event: evt
		selectors: {#click:. #doubleClick:. nil. #startDrag:}
		threshold: 1. ! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:49:23.957'!
mouseLeave: evt

	self hoveredCell: nil.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:49:27.815'!
mouseMove: evt

	self hoveredCell: (self cellIndexFromPoint: evt position).! !

!UiTreeViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/11/2010 09:17:00.054'!
rowBoundsFor: aRowIndex

	| row |
	row := rows at: aRowIndex.
	
	^ self left @ (self topForRow: row)
		extent: self width @ row height! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/10/2010 20:03:34.835'!
selection

	^ selection! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:12:39.162'!
selection: aSelection

	selection := aSelection.
	self connect: aSelection signal: #selectionChanged toSelector: #redrawLater.
	
	self redrawLater.! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/10/2010 20:13:12.402'!
selectionBehavior

	^ selectionBehavior ifNil: [selectionBehavior := #selectRows]! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:28:20.274'!
selectionBehavior: aSymbol
	"#selectCells, #selectRows, #selectColumns"

	selectionBehavior := aSymbol.
	self redrawLater.! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 14:30:17.153'!
selectionColor

	^ selectionColor ifNil: [selectionColor := Color r: 0.772 g: 0.858 b: 1.0]! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/8/2010 14:37:57.041'!
selectionColor: aColor

	selectionColor := aColor.! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:04:47.949'!
selectionMode

	^ selectionMode! !

!UiTreeViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:04:07.606'!
selectionMode: aSelectionClass

	selectionMode := aSelectionClass.
	self selection: aSelectionClass new.! !
