
Morph subclass: #UiTreeViewport
	instanceVariableNames: 'rows columns selectionColor selectedIndex selectionMode indent verticalHeader horizontalHeader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/10/2010 11:14:04.16'!
drawOn: aCanvas
	
	self
		drawSelectionOn: aCanvas;
		drawTreeStructureOn: aCanvas.
	super drawOn: aCanvas.! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/9/2010 12:19:06.43'!
drawSelectionOn: aCanvas
	
	self selectedIndex ifNil: [^ self].
	
	self selectionMode = #singleCell ifTrue: [
		^ self
			drawSelectionOn: aCanvas
			bounds: (self cellBoundsFor: self selectedIndex)].
		
	self selectionMode = #singleRow ifTrue: [
		^ self
			drawSelectionOn: aCanvas
			bounds: (self rowBoundsFor: self selectedIndex)].! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/8/2010 14:38:54.229'!
drawSelectionOn: aCanvas bounds: drawBounds
	
	| gradient |

	gradient := (GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (self selectionColor adjustBrightness: 0.05).
			0.6 -> (self selectionColor darker)})
		origin: drawBounds topLeft;
		direction: 0 @ drawBounds height.
		
	aCanvas
		fillRoundRect: (drawBounds insetBy: (1@1 corner: 1@1))
		radius: 3
		fillStyle: gradient;
		frameRoundRect: drawBounds
		radius: 3
		width: 1
		color: self selectionColor muchDarker.! !

!UiTreeViewport methodsFor: 'initialization' stamp: 'mt 10/9/2010 12:48:33.796'!
initialize

	super initialize.
	self color: Color transparent.
		
	rows := OrderedCollection new.
	columns := OrderedCollection new.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/9/2010 12:59:15.866'!
mouseDown: evt

	self selectedIndex: (self cellIndexFromPoint: evt position).

	evt hand
		waitForClicksOrDrag: self
		event: evt
		selectors: {#click:. #doubleClick:. nil. #startDrag:}
		threshold: 1. "pixel threshold"! !

!UiTreeViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/9/2010 12:18:03.687'!
rowBoundsFor: aCellIndex

	| row |
	row := rows at: aCellIndex y.
	
	^ self left @ (self topForRow: row)
		extent: self width @ row height! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/8/2010 14:39:55.887'!
selectedIndex

	^ selectedIndex! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/9/2010 13:05:17.511'!
selectedIndex: aCellIndex

	selectedIndex := aCellIndex.
	self redrawLater.
	self selectionChanged: (self modelIndexFor: aCellIndex).! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 10:57:51.587'!
selectionColor

	^ selectionColor ifNil: [selectionColor := Color r: 0.721 g: 0.815 b: 0.972]! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/8/2010 14:37:57.041'!
selectionColor: aColor

	selectionColor := aColor.! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/9/2010 20:20:15.303'!
selectionMode

	^ selectionMode ifNil: [selectionMode := #singleRow]! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/8/2010 15:09:45.981'!
selectionMode: aMode
	"#singleCell, #singleRow, #connectedCells, #connectedRows, #multipleCells, #multipleRows"

	selectionMode := aMode.
	self redrawLater.! !

!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/10/2010 14:34:51.957'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; width: self scrollBarThickness; changeTableLayout; listDirection: #topToBottom.
	horizontalHeader := Morph new color: Color transparent; height: self scrollBarThickness; changeTableLayout; listDirection: #leftToRight.
	vHeaderScroller := TransformMorph new color: Color transparent.
	hHeaderScroller := TransformMorph new color: Color transparent.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: UiTreeViewport new.
	self connect: self viewport signal: #doubleClicked: toSelector: #toggleIndex:.
	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	itemDelegates at: #default put: UiSimpleItemDelegate new.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller addMorph: horizontalHeader.

	self
		"addMorph: vHeaderScroller;"
		addMorph: hHeaderScroller.
	
	self resizeScroller.! !
